#!/usr/bin/perl -w


  use warnings;
  use strict;
  use subprogramQDD;
#  use Data::Dumper;
  
  
#INPUT: xxx_pipe3.fas file with clean (no adapter, no tag) microsatellite containing non-redundant sequences (output of pipe2.pl) in a data3_xxx folder
# output: 

# if $direct_par_def = 0 (command line menu) set_file is read and the user is asked if the settings are correct and can change them 
# if $direct_par_def = 1 all parameters are defined in the program; 
# if $direct_par_def = 2 the set_file is read and the user cannot change the parameters (option for running behind GUI)

my $direct_par_def = 0;

my $mask_mono = 5;
my $mask_di = 3;
my $mask_tri = 3;
my $mask_tetra = 3;
my $mask_penta = 3;
my $mask_hexa = 3;
my $set_file = 'set_pipe3.txt';
my $set_file_default = 'set_pipe3_default.txt';

print_heading();

my %param = (
  syst => '',  
  primer3_path => '',
  primer3_version => '',
  project_folder => '',
  pcr_min => '',
  pcr_max => '',
  pcr_step => '',
  PRIMER_GC_CLAMP => '',
  PRIMER_OPT_SIZE => '',
  PRIMER_MIN_SIZE => '',
  PRIMER_MAX_SIZE => '',
  PRIMER_OPT_TM => '',
  PRIMER_MIN_TM => '',
  PRIMER_MAX_TM => '',
  PRIMER_MAX_DIFF_TM => '',
  PRIMER_MIN_GC => '',
  PRIMER_OPT_GC_PERCENT => '',
  PRIMER_MAX_GC => '',
  PRIMER_SELF_ANY => '',
  PRIMER_SELF_END => '',
  PRIMER_MAX_POLY_X => '',
  PRIMER_NUM_RETURN => '',
  del_files => ''
);

my %param_list = define_param_number_pipe3(); # $param_list{1}[syst, 'Operating system (win/linux):'], ...
my %param_limits = define_param_limits_pipe3(\%param); #hash of array $param_limits{syst}[type(list, real, folder), error message, ['list of values/lower_upper limits)']

if ($direct_par_def == 1) # define parameteres values in the script
{
  $param{syst} = 'win';  
  $param{primer3_path} = 'c:/primer3-1.1.4/bin/';
  $param{primer3_version} = 1;
  $param{project_folder} = 'd:/test with space';
  $param{del_files} = 1;
  $param{pcr_min} = 90;
  $param{pcr_max} = 300;
  $param{pcr_step} = 210;
  $param{PRIMER_GC_CLAMP} = 0;
  $param{PRIMER_MIN_SIZE} = 18;
  $param{PRIMER_MAX_SIZE} = 27;
  $param{PRIMER_OPT_SIZE} = 20;
  $param{PRIMER_OPT_TM} = 60.6;
  $param{PRIMER_MIN_TM} = 57.0;
  $param{PRIMER_MAX_TM} = 63.0;
  $param{PRIMER_MAX_DIFF_TM} = 10.0; # default Primer3 100.0
  $param{PRIMER_MIN_GC} = 20.0;
  $param{PRIMER_OPT_GC_PERCENT} = 50.0;
  $param{PRIMER_MAX_GC} = 80.0;
  $param{PRIMER_SELF_ANY} = 8.0;
  $param{PRIMER_SELF_END} = 3.0;
  $param{PRIMER_MAX_POLY_X} = 3; # default Primer3 5
  $param{PRIMER_NUM_RETURN} = 3; # default Primer3 5
}
elsif($direct_par_def == 2) # for GUI
{
  read_set_file_for_GUI(\%param, $set_file, $set_file_default);
}
elsif($direct_par_def == 0)
{
  read_set_file_for_command_line(\%param, $set_file, \%param_list, $set_file_default);
}

check_param_limits(\%param, \%param_limits);

#determine the last created folder within $project_folder, that contains motif ('^data3');
my $pipe2_folder = determine_last_folder($param{project_folder}, '^pipe2');
my %pipe2_list = get_file_folder_list($pipe2_folder);
my $outfolder = time;
make_folder($param{syst}, $outfolder);

my $date = localtime $outfolder;

print "Input pipe2 folder: $pipe2_folder\n";
my $pipe3_folder = $param{project_folder}.'/pipe3_'.$outfolder;
make_folder($param{syst}, $pipe3_folder);
print "Output pipe3 folder: $pipe3_folder\n";
my $log_file = $pipe3_folder.'/pipe3_log_'.$outfolder.'.txt';

  open_file_out($log_file, 'LOG');
  print LOG "**********************************************************\n";
  print LOG "QDD version2.1(beta) 28 June 2011\n";
  print LOG "Emese Meglecz, Aix-Marseille University, Marseille, France\n";
  print LOG 'emese.meglecz@univ-provence.fr';
  print LOG "\n";
  print LOG "Plesae, read the Documentation_QDD2.pdf\n";
  print LOG "**********************************************************\n\n";
  print LOG "Output folder identifyer: $outfolder\n";
  print LOG "Run started at : $date\n";
  print LOG "**********************************************************\n";
  print LOG "Parameters:\nSystem: $param{syst}\n";
  print LOG "Minimum homopolymer length for soft masking: $mask_mono\n";
  print LOG "Minimum repeat number for dibase motifs for sof masking: $mask_di\n";
  print LOG "Minimum repeat number for tribase motifs for sof masking: $mask_tri\n";
  print LOG "Minimum repeat number for tetrabase motifs for sof masking: $mask_tetra\n";
  print LOG "Minimum repeat number for pentabase motifs for sof masking: $mask_penta\n";
  print LOG "Minimum repeat number for hexabase motifs for sof masking: $mask_hexa\n";
  print LOG "Path to Primer3 executables: $param{primer3_path}\n";
  print LOG "Primer3 version(1/2): $param{primer3_version}\n";
  print LOG "Project folder: $param{project_folder}\n";
  print LOG "Pipe2 folder (input): $pipe2_folder\n";
  print LOG "Minimum PCR product size: $param{pcr_min}\n";
  print LOG "Maximum PCR product size: $param{pcr_max}\n";
  print LOG "PCR size step: $param{pcr_step}\n";
  print LOG "PRIMER_GC_CLAMP: $param{PRIMER_GC_CLAMP}\n";
  print LOG "PRIMER_OPT_SIZE: $param{PRIMER_OPT_SIZE}\n";
  print LOG "PRIMER_MIN_SIZE: $param{PRIMER_MIN_SIZE}\n";
  print LOG "PRIMER_MAX_SIZE: $param{PRIMER_MAX_SIZE}\n";
  print LOG "PRIMER_OPT_TM: $param{PRIMER_OPT_TM}\n";
  print LOG "PRIMER_MIN_TM: $param{PRIMER_MIN_TM}\n";
  print LOG "PRIMER_MAX_TM: $param{PRIMER_MAX_TM}\n";
  print LOG "PRIMER_MAX_DIFF_TM: $param{PRIMER_MAX_DIFF_TM}\n";
  print LOG "PRIMER_MIN_GC: $param{PRIMER_MIN_GC}\n";
  print LOG "PRIMER_OPT_GC_PERCENT: $param{PRIMER_OPT_GC_PERCENT}\n";
  print LOG "PRIMER_MAX_GC: $param{PRIMER_MAX_GC}\n";
  print LOG "PRIMER_SELF_ANY: $param{PRIMER_SELF_ANY}\n";
  print LOG "PRIMER_SELF_END: $param{PRIMER_SELF_END}\n";
  print LOG "PRIMER_MAX_POLY_X: $param{PRIMER_MAX_POLY_X}\n";
  print LOG "PRIMER_NUM_RETURN: $param{PRIMER_NUM_RETURN}\n";
  print LOG "**********************************************************\n";


my $trans_file = 'motifs_transmot_hach1_6.txt';
my %trans = open_file_and_hach($trans_file, '1');
  # make lc motifs in %trans
foreach my $mot (keys %trans)
{
  my $mot_lc = lc $mot;
  $trans{$mot_lc} = $trans{$mot};
}


foreach my $file (sort keys %pipe2_list)
{

if ($file =~ /_pipe3\.fas/)
{  
  my $time_beg = time;

  my $file_root = $file;
  $file_root =~ s/\.fas//;
  
  my $outfolder_spec = '';
  $outfolder_spec = $outfolder.'/'.$file_root.'_'.$outfolder;
  make_folder($param{syst}, $outfolder_spec);

  my $primer_file = $pipe3_folder.'/'.$file_root.'_primers.csv';
  my $primer_target_seq_file = $pipe3_folder.'/'.$file_root.'_targets.fas';
  my $primer3_inp_file = $outfolder_spec.'/'.$file_root.'_primer3_input.txt';
  my $primer3_out_file = $outfolder_spec.'/'.$file_root.'_primer3_output.txt';

  my %seq_count = ();
  
  $file = $pipe2_folder.'/'.$file;
  print LOG "\n************************\n$file\n************************\n";
  print "\n*****************************\n$file\n";
  
  # %polym is a hash of tables with cons seq code as keys and a table of motif_fpos_lastpos as value
  my %polym = make_polymorph_info($file);
  my $seq_file = '';
if (1) # mask first microsatellites, keep mask then mask all micro- nanosatellites and homopomlymers at the same time : TTTTTTTGTGTGTGTG: T6(TG)5
{
# mask only target MS in lc. In the next find MS(1) target MS are already masked and will not be taken as part of a shorter MS just before 
  my $find_ms0 = 'perl find_and_mask_ms2.pl 1000000 5 5 5 5 5 1 1 "'.$file.'" "'.$outfolder_spec.'" pipe3\.fas 0 1 1';
  $file = $outfolder_spec.'/'.$file_root.'_mask.fas';
	print LOG "\n$find_ms0\n";
  print "Soft masking sequences\n";
  system $find_ms0;

    # mask MS and homopolymers in the input fasta file use low limit values to avoid false BLAST hits  and eliminate CSS sequences (no autohit)
  my $find_ms1 = 'perl find_and_mask_ms2.pl '.$mask_mono.' '.$mask_di.' '.$mask_tri.' '.$mask_tetra.' '.$mask_penta.' '.$mask_hexa.' 0 1 "'.$file.'" "'.$outfolder_spec.'" pipe3_mask\.fas 1 1 1';
    #  USAGE: $min1 $min2 $min3 $min4	$min5	$min6	$del_prev_mask $keep_non_ms $folder/$file	$outfolder $file_motif $write_seq_file $write_fas_file $one_file;
	print LOG "\n$find_ms1\n";
#  print "Soft masking sequences\n";
  system $find_ms1;
 
  $file = $outfolder_spec.'/'.$file_root.'_mask_mask.fas';
  $seq_file = $outfolder_spec.'/'.$file_root.'_mask.seq';
}# end if()
  
if (0) # mask all micro- nanosatellites and homopomlymers at the same time : TTTTTTTGTGTGTGTG: T7(GT)4G
{
      # mask MS and homopolymers in the input fasta file use low limit values to avoid false BLAST hits  and eliminate CSS sequences (no autohit)
  my $find_ms1 = 'perl find_and_mask_ms2.pl '.$mask_mono.' '.$mask_di.' '.$mask_tri.' '.$mask_tetra.' '.$mask_penta.' '.$mask_hexa.' 1 1 "'.$file.'" "'.$outfolder_spec.'" pipe3\.fas 1 1 1';
    #  USAGE: $min1 $min2 $min3 $min4	$min5	$min6	$del_prev_mask $keep_non_ms $folder/$file	$outfolder $file_motif $write_seq_file $write_fas_file $one_file;
	print LOG "\n$find_ms1\n";
#  print "Soft masking sequences\n";
  system $find_ms1;
 
  $file = $outfolder_spec.'/'.$file_root.'_mask.fas';
  $seq_file = $outfolder_spec.'/'.$file_root.'.seq';
} # end if ()

  # %ms is a hash of table of tables; hash keys are seq codes $ms{$code} that contain a table of pooled microsatellites$ms{$code}[ms]
  # the first element of the ms table is the list of the first positions of target microsatellites
  # Then for each microsatellite either [first_pos, last_pos, c/p, repeat_number(of the longest pure sterch), $motif(of the longest pure sterch), $repeat_length(of last strech; deleted after finishing the %ms)]
  #Homopolymers are never pooled to any other MS.
  # Ms are pooled if the distance between them is not greater than the longest motif length of the two neigbouring MS
  my %ms =  make_ms_hash($seq_file);
 

if (0)
{
  open_file_out($outfolder_spec.'/temp.txt', 'TMP');
  print TMP Dumper (\%ms);
  close TMP;
}
  
  my %seq = read_hach_from_fasta_code_wo_space($file);
  $seq_count{'Total number of input sequences:'} = scalar keys%seq;
  delete_no_target_seq(\%seq, \%ms); # delete sequence from %seq that do not have taget MS (rare cases like tttttgtgtgtgtG)
  $seq_count{'Total number of sequences with target MS:'} = scalar keys%seq;

  # primer design conditions:
  #   homo  nano_primer nano_flank  other_target_in flank target_comp
  # A no    no          no          no                    no  
  # B no    no          yes         no                    yes  
  # C no    yes         yes         no                    yes 
  # D no    no          yes         yes                   yes  
  # E no    yes         yes         yes                   yes 
  # F yes   yes         yes         yes                   yes 

  my %primer3_base = ();
  # $$primer3_base{$code.'_'.$target_ms}{'seq'} = uc$$seq{$code};
  # $$primer3_base{$code.'_'.$target_ms}{'target'} = $target; $first_pos,length
  # $$primer3_base{$code.'_'.$target_ms}{'excluded'} = $excluded; reference to an anonymous table of $first_pos,length
  my %results = (); #$results{$code_seq}{left_pos,left_l,rigth_pos,rigth_l}{left_seq=>, ...}  
  my %target_ms_info = (); # $target_ms{secode_first_pos} = [first_pos, length_bp, length_in_rn, p/c, mot_trans, ms_seq ];

  my @condition_of_design = ('A','B','C','D','E','F','G');
#  my @condition_of_design = ('A','B','C','D');
  foreach my $condition (@condition_of_design)
  {
    print "Identifying target and excluded regions with $condition design\n";
    select_target_excluded($condition, \%seq, \%ms, \%primer3_base, \%target_ms_info, \%trans); # %primer3_base is emptyed at the begining of the subroutine
 
    for(my $min= $param{pcr_min}; $min<$param{pcr_max}; $min = $min+$param{pcr_step}) # run primer3 for each PCR product length step
    {

      my $max = $min + $param{pcr_step}-1;
      if ($max > $param{pcr_max})
      {
        $max = $param{pcr_max};
      }
      print "Running Primer3 with $min - $max PCR product interval\n";

      write_primer3_input($primer3_inp_file, \%primer3_base, $min, $max, $param{PRIMER_GC_CLAMP}, $param{PRIMER_OPT_SIZE}, $param{PRIMER_MIN_SIZE}, $param{PRIMER_MAX_SIZE}, $param{PRIMER_OPT_TM}, $param{PRIMER_MIN_TM}, $param{PRIMER_MAX_TM}, $param{PRIMER_MAX_DIFF_TM}, $param{PRIMER_MIN_GC}, $param{PRIMER_OPT_GC_PERCENT}, $param{PRIMER_MAX_GC}, $param{PRIMER_SELF_ANY}, $param{PRIMER_SELF_END}, $param{PRIMER_MAX_POLY_X}, $param{PRIMER_NUM_RETURN});
      unless (-z $primer3_inp_file) # if file is not empty
      {
        run_primer3($param{syst}, $primer3_inp_file, $primer3_out_file, $param{primer3_path});
        read_results($primer3_out_file, \%results, $condition, \%target_ms_info);
      }
    }
  
  }
  
  if (0)
  {
    open_file_out($outfolder_spec.'/pri.txt', 'TMP');
    print TMP Dumper(\%results);
    close TMP;
  }

  $seq_count{'Total number of sequences with primers:'} = scalar keys%results;
  make_fasta_file_with_designed_seq(\%results, \%seq, $primer_target_seq_file);
  write_primer_table(\%results, \%target_ms_info, $primer_file, \%polym, \%seq);
  
  print LOG "\n\n";
  foreach my $code (sort keys %seq_count)
  {
    print LOG "$code $seq_count{$code}\n";
  }
  my $time_end = time;
  my $time_run = $time_end - $time_beg;
  print LOG "\nThe analyses of $file_root.fas took $time_run s.\n";
  
  if ($param{del_files})
  {
    delete_folder($outfolder_spec,$param{syst});
  }

  
} # end if file motif
} # end foreach my $file

close LOG;

if ($param{del_files}==1)
{
  delete_folder($outfolder,$param{syst});
}

print "\nThe anlysis is finished\n";

exit;

############################################################


sub make_ms_hash
{
my ($seq_file) = @_;
my %ms = ();
# %ms is a hash of table of tables; hash keys are seq codes $ms{$code} that contains a table of pooled microsatellites$ms{$code}[ms]
# the first element of the ms table is the list of the first positions of target microsatellites
# Then for each microsatellite either [first_pos, last_pos, c/p, repeat_number(of the longest pure sterch), $motif(of the longest pure sterch), $repeat_length(of last strech; deleted after finishing the %ms)]
#Homopolymers are never pooled to any other MS.
# Ms are pooled if the distance between them is not greater than the longest motif length of the two neigbouring MS

open_file_in($seq_file, 'SEQ');
my $title = <SEQ>;
while(my $line = <SEQ>)
{
  chomp $line;
  $line =~ s/;$//;
  my @line = split(';', $line);
  # initialize first element of the table (this will be the list of the first positions of target MS)
    $ms{$line[0]} = [[0]];
  for (my $i= 3; $i<scalar@line; $i =$i+4)
  {
    if (scalar@{${$ms{$line[0]}}[-1]} == 6 and length$line[$i]>1) # if there is a MS before and neither this nor the actual Ms is homopolymer 
    {
      my $dist_ms = $line[$i+1] - $ms{$line[0]}[-1][1] - 1;
      if ( ($dist_ms > $ms{$line[0]}[-1][5]) and ($dist_ms > length$line[$i])) # if the actual and previous MS have greater than the (longest motif length) bases between
      {
#        print "$line[0], $line[$i], $line[$i+1], $line[$i+2], $line[$i+3], $dist_ms, @{${$ms{$line[0]}}[-1]}\n";
        new_ms($ms{$line[0]}, $line[$i], $line[$i+1], $line[$i+2], $line[$i+3]);
      }
      else #distance is small between neigbouring non-homopolymer MS
      {
        pool_ms($ms{$line[0]}, $line[$i], $line[$i+1], $line[$i+2], $line[$i+3]);
      }
    }
    else # first MS in seq, or previois or actual ms is homoploymer
    {
      new_ms($ms{$line[0]}, $line[$i], $line[$i+1], $line[$i+2], $line[$i+3]);
    }
  } # end for
  # delete the motif lenght of the last strech
  for(my $j= 1; $j< scalar @{$ms{$line[0]}}; ++$j)
  {
    if (scalar @{$ms{$line[0]}[$j]} == 6)
    {
      pop@{$ms{$line[0]}[$j]};
    }
  }

  # delete seq if there is no target MS

  shift@{$ms{$line[0]}[0]};
  unless (scalar @{$ms{$line[0]}[0]})
  {
    delete $ms{$line[0]};
    print "$line[0]\n";
  }

}
close SEQ;


return %ms; 
}
#####################################################

sub pool_ms
{
my ($code_table_ref, $motif, $beg, $end, $rep_n) = @_;

$$code_table_ref[-1][1] = $end;
$$code_table_ref[-1][2] = 'c';
if ($rep_n > $$code_table_ref[-1][3])
{
  $$code_table_ref[-1][3] = $rep_n;
  $$code_table_ref[-1][4] = $motif;
}
  $$code_table_ref[-1][5] = length$motif;

if ($rep_n > 4)
{
  unless( $$code_table_ref[0][-1] == $$code_table_ref[-1][0])
  {
    push(@{$$code_table_ref[0]}, $$code_table_ref[-1][0]);    
  }
}

}
#####################################################

sub new_ms
{
my ($code_table_ref, $motif, $beg, $end, $rep_n) = @_;

my $lm = length $motif;

if ($lm ==1)
{
  push(@{$code_table_ref}, [$beg, $end]);
} 
else
{
  push(@{$code_table_ref}, [$beg, $end, 'p', $rep_n, $motif, $lm]);
  if($rep_n > 4)
  {
    push(@{$$code_table_ref[0]}, $beg);
  }
}

}
#########################################################

sub delete_no_target_seq
{
my ($seq, $ms) = @_;
foreach my $code (keys %$seq)
{
  unless(exists $$ms{$code})
  {
    delete $$seq{$code};
  }
}
}

#########################################################

sub select_target_excluded
{
my ($condition, $seq, $ms, $primer3_base, $target_ms_info, $trans) = @_;
%$primer3_base = ();
# $$primer3_base{$code.'_'.$target_ms}{'seq'} = uc$$seq{$code};
# $$primer3_base{$code.'_'.$target_ms}{'target'} = $target; $first_pos,length
# $$primer3_base{$code.'_'.$target_ms}{'excluded'} = $excluded; reference to an anonymous table of $first_pos,length
# %target_ms_info = (); # $target_ms{seqcode_first_pos} = [$first_pos, $length_bp, $length_rn, p/c, $seqfr, mot_trans];

# %ms is a hash of table of tables; $ms{$code}[ms][first_pos, last_pos, c/p, repeat_number, $motif] or [first_pos, last_pos]
# hash keys are seq codes $ms{$code} that contains a table of pooled microsatellites$ms{$code}[ms]
# the first element of the ms table is the list of the first positions of target microsatellites
# Then for each microsatellite either [first_pos, last_pos, c/p, repeat_number(of the longest pure sterch), $motif(of the longest pure sterch), $repeat_length(of last strech; deleted after finishing the %ms)]
#Homopolymers are never pooled to any other MS.
# Ms are pooled if the distance between them is not greater than the longest motif length of the two neigbouring MS

  # primer design conditions:
  #   homo  nano_primer nano_flank  other_target_in flank target_comp
  # A no    no          no          no                    no  
  # B no    no          yes         no                    yes  
  # C no    yes         yes         no                    yes 
  # D no    no          yes         yes                   yes  
  # E no    yes         yes         yes                   yes 
  # F yes   yes         yes         yes                   yes 
  
foreach my $code (sort keys %$ms) # for all sequences
{
  my $target_l = '';
  my $target = '';
  my $excluded = [];
  if ($condition eq 'A' or $condition eq 'B' or $condition eq 'C' or $condition eq 'F') # only one target MS
  {
  for ( my $t = 0; $t < scalar@{$$ms{$code}[0]}; ++$t) # for all target ms
  {
    my $target_ms = $$ms{$code}[0][$t];
    for (my $i = 1; $i< scalar@{$$ms{$code}}; ++$i) # identify target MS info in table
    {
      if ($$ms{$code}[$i][0] == $target_ms) # identify target MS info in table
      {      
        $target_l = $$ms{$code}[$i][1] - $$ms{$code}[$i][0] +1;
        $target = $$ms{$code}[$i][0].','.$target_l;
        $excluded = [];
        if ($condition eq 'A')
        {
          if (my $bool = design_a($code, $i, $ms, $seq, $excluded))
          {
            fill_primer3_base($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $target_ms, $target_l, $code, $i, $i);
          }
        }
        elsif ($condition eq 'B' or $condition eq 'C') 
        {
          design_bc($code, $i, $ms, $seq, $excluded, $condition);
          fill_primer3_base($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $target_ms, $target_l, $code, $i, $i);
        }
        elsif ($condition eq 'F')
        {
          design_fg($code, $ms, $seq, $excluded, $i, $i);
          fill_primer3_base($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $target_ms, $target_l, $code, $i, $i);
        }
        $i = scalar@{$$ms{$code}};
      } # end if
    } # end for $i
  }# end for $t
  } #end if $condition ABC 
  elsif (scalar@{$$ms{$code}[0]} > 1)  # codititons DEG and more than 1 target allowed in target region
  {
    my $f_target_index = '';
    my $l_target_index = '';
    for (my $t1 = 0; $t1 <scalar@{$$ms{$code}[0]}-1; ++$t1) # determine all possible pairs of silge target to make a coumpound target region
    {
      for (my $t2 = $t1+1; $t2 <scalar@{$$ms{$code}[0]}; ++$t2) # determine all possible pairs of silge target to make a coumpound target region
      {
        for (my $i = 1; $i< scalar@{$$ms{$code}}; ++$i) # identify target MS info in table
        {
          if ($$ms{$code}[$i][0] == $$ms{$code}[0][$t1]) # identify first target MS info in table
          {
            $f_target_index = $i;
          }
          elsif ($$ms{$code}[$i][0] == $$ms{$code}[0][$t2]) # identify last target MS info in table
          {
            $l_target_index = $i;
          }
        }# end for $i
        if ($condition eq 'D' or $condition eq 'E') # next pair if there is a homopolymer between the target ms and disign is ED
        {
          my $homo_bool = check_homopolymer_between_targets($ms, $code, $f_target_index, $l_target_index);
          if ($homo_bool) # homopolymer between target mss
          {
            $t2 = scalar@{$$ms{$code}};
            ++$t1;
            next;
          }
        } # if ($condition eq 'D' or $condition eq 'E')
        
        $target_l = $$ms{$code}[$l_target_index][1] - $$ms{$code}[$f_target_index][0] +1;
        $target = $$ms{$code}[$f_target_index][0].','.$target_l;
        $excluded = [];
                 
        if ($condition eq 'D' or $condition eq 'E')
        {
          design_de($code, $ms, $seq, $excluded, $f_target_index, $l_target_index, $condition);
          fill_primer3_base($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $$ms{$code}[$f_target_index][0], $target_l, $code, $f_target_index, $l_target_index);
        }
        elsif ($condition eq 'G')
        {
          design_fg($code, $ms, $seq, $excluded, $f_target_index, $l_target_index);
          fill_primer3_base($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $$ms{$code}[$f_target_index][0], $target_l, $code,  $f_target_index, $l_target_index);
        }

        
      } # end for $T2 
    }# end for $1
  }# end elsif (scalar@{$$ms{$code}[0] > 1)  # codititons DEF and more than 1 target
}# end foreach


}

##############################################

sub check_homopolymer_between_targets
{
my ($ms, $code, $f_index, $l_index) = @_;
my $bool = 0;

for (my $i = ($f_index +1); $i < $l_index; ++$i)
{
  if (scalar@{$$ms{$code}[$i]} ==2)
  {
    $bool = 1;
    last;
  }
}

return $bool;
}
################################################
sub fill_primer3_base
{
my ($primer3_base, $target_ms_info, $trans, $ms, $seq, $excluded, $target_ms, $target_l, $code, $f_target_index, $l_target_index) = @_;

#my @tagget_list = split(',', $target_ms);

 $$primer3_base{$code.'_'.$target_ms.','.$target_l}{'seq'} = uc$$seq{$code};
 $$primer3_base{$code.'_'.$target_ms.','.$target_l}{'target'} = $target_ms.','.$target_l; 
 $$primer3_base{$code.'_'.$target_ms.','.$target_l}{'excluded'} = $excluded;
 unless(exists $$target_ms_info{$code.'_'.$target_ms.','.$target_l})
 {
  my $seq_ms = substr(uc$$seq{$code}, $target_ms-1, $target_l);
  my $longest_ms = 0;
  my $longest_ms_index = 0;
  for(my $i = $f_target_index; $i<=$l_target_index; ++$i)
  {
    if (scalar@{$$ms{$code}[$i]} > 2 and $$ms{$code}[$i][3] > $longest_ms)
    {
      $longest_ms = $$ms{$code}[$i][3];
      $longest_ms_index = $i;
    }
  }
  
  my  $mot_trans = $$trans{$$ms{$code}[$longest_ms_index][4]};
  $$target_ms_info{$code.'_'.$target_ms.','.$target_l} = [$target_ms, $target_l, $$ms{$code}[$longest_ms_index][3], $$ms{$code}[$longest_ms_index][2], $mot_trans, $seq_ms];
 }
}
##################################################


sub design_a
{
my ($code, $i, $ms, $seq, $excluded) = @_;
my $bool = 0; 
if ($$ms{$code}[$i][2] eq 'p')
{ 
    $bool = 1;
    if ($i > 1)
    {
      push(@$excluded, '1,'.$$ms{$code}[$i-1][1])
    }
    if ($i+1 < scalar @{$$ms{$code}})
    {
      push(@$excluded, $$ms{$code}[$i+1][0].','.((length$$seq{$code}) - $$ms{$code}[$i+1][0] +1))
    }
}
return $bool;
}
###########################################

sub design_bc
{
my ($code, $i, $ms, $seq, $excluded, $condition) = @_;
# $ms{$code}[ms][first_pos, last_pos, c/p, repeat_number, $motif] or [first_pos, last_pos]

foreach (my $j = $i-1; $j > 0; $j = $j-1) 
{
  if (scalar@{$$ms{$code}[$j]} == 2 or $$ms{$code}[$j][3] > 4) # find the the closest nano of target ms before the tagret ms 
  {
      push(@$excluded, '1,'.$$ms{$code}[$j][1]);
      last;
  }
  if ($condition eq 'B')
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat
  }
}

foreach (my $j = $i+1; $j < scalar @{$$ms{$code}}; ++$j)
{
  if (scalar@{$$ms{$code}[$j]} == 2 or $$ms{$code}[$j][3] > 4)  # find the the closest nano of target ms after the tagret ms 
  {
      push(@$excluded, $$ms{$code}[$j][0].','.((length$$seq{$code}) - $$ms{$code}[$j][0] +1));
      last;
  }
  if ($condition eq 'B')
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat
  }
}

}

################################################

sub design_de
{
my ($code, $ms, $seq, $excluded, $f_target_index, $l_target_index, $condition) = @_;
# $ms{$code}[ms][first_pos, last_pos, c/p, repeat_number, $motif] or [first_pos, last_pos]

foreach (my $j = $f_target_index-1; $j > 0; $j = $j-1) 
{
  if (scalar@{$$ms{$code}[$j]} == 2) # find the closest homopol of target ms before the tagret ms 
  {
      push(@$excluded, '1,'.$$ms{$code}[$j][1]);
      last;
  }
  elsif ($$ms{$code}[$j][3] > 4)
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat and other target ms  
  }
  elsif ($condition eq 'D')
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat and other target ms
  }
}
foreach (my $j = $l_target_index+1; $j < scalar @{$$ms{$code}}; ++$j)
{
  if (scalar@{$$ms{$code}[$j]} == 2)  # find the closest homopol of target ms after the tagret ms 
  {
      push(@$excluded, $$ms{$code}[$j][0].','.((length$$seq{$code}) - $$ms{$code}[$j][0] +1));
      last;
  }
  elsif ($$ms{$code}[$j][3] > 4)
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat and other target ms  
  }
  elsif ($condition eq 'D')
  {
    push(@$excluded, $$ms{$code}[$j][0].','.($$ms{$code}[$j][1] - $$ms{$code}[$j][0] +1)); # exclude nanosat and other target
  }
}
}

################################################

sub design_fg
{
my ($code, $ms, $seq, $excluded, $f_target_index, $l_target_index) = @_;
# $ms{$code}[ms][first_pos, last_pos, c/p, repeat_number, $motif] or [first_pos, last_pos]

foreach (my $j = $f_target_index-1; $j > 0; $j = $j-1) 
{
  if (scalar@{$$ms{$code}[$j]} > 2 and $$ms{$code}[$j][3] > 4) # find the closest target ms before the tagret region 
  {
      push(@$excluded, '1,'.$$ms{$code}[$j][1]);
      last;
  }
}
foreach (my $j = $l_target_index+1; $j < scalar @{$$ms{$code}}; ++$j)
{
  if (scalar@{$$ms{$code}[$j]} > 2 and $$ms{$code}[$j][3] > 4)  # find the closest target ms after the tagret region 
  {
      push(@$excluded, $$ms{$code}[$j][0].','.((length$$seq{$code}) - $$ms{$code}[$j][0] +1));
      last;
  }
}
}
################################################


sub write_primer3_input
{
my ($primer3_inp_file, $primer3_base, $min, $max, $PRIMER_GC_CLAMP, $PRIMER_OPT_SIZE, $PRIMER_MIN_SIZE, $PRIMER_MAX_SIZE, $PRIMER_OPT_TM, $PRIMER_MIN_TM, $PRIMER_MAX_TM, $PRIMER_MAX_DIFF_TM, $PRIMER_MIN_GC, $PRIMER_OPT_GC_PERCENT, $PRIMER_MAX_GC, $PRIMER_SELF_ANY, $PRIMER_SELF_END, $PRIMER_MAX_POLY_X, $PRIMER_NUM_RETURN) = @_;  

open_file_out($primer3_inp_file, 'PR');
foreach my $code (sort keys %$primer3_base)
{
  print PR "PRIMER_SEQUENCE_ID=$code\n";
  print PR "SEQUENCE=$$primer3_base{$code}{'seq'}\n";
  print PR "PRIMER_FIRST_BASE_INDEX=1\n";
  print PR "TARGET=$$primer3_base{$code}{'target'}\n";
  print PR "PRIMER_GC_CLAMP=$PRIMER_GC_CLAMP\n";
  print PR "PRIMER_OPT_SIZE=$PRIMER_OPT_SIZE\n";
  print PR "PRIMER_MIN_SIZE=$PRIMER_MIN_SIZE\n";
  print PR "PRIMER_MAX_SIZE=$PRIMER_MAX_SIZE\n";
  print PR "PRIMER_OPT_TM=$PRIMER_OPT_TM\n";
  print PR "PRIMER_MIN_TM=$PRIMER_MIN_TM\n";
  print PR "PRIMER_MAX_TM=$PRIMER_MAX_TM\n";
  print PR "PRIMER_MAX_DIFF_TM=$PRIMER_MAX_DIFF_TM\n";
  print PR "PRIMER_MIN_GC=$PRIMER_MIN_GC\n";
  print PR "PRIMER_OPT_GC_PERCENT=$PRIMER_OPT_GC_PERCENT\n";
  print PR "PRIMER_MAX_GC=$PRIMER_MAX_GC\n";
  print PR "PRIMER_SELF_ANY=$PRIMER_SELF_ANY\n";
  print PR "PRIMER_SELF_END=$PRIMER_SELF_END\n";
  print PR "PRIMER_MAX_POLY_X=$PRIMER_MAX_POLY_X\n";
  print PR "PRIMER_NUM_RETURN=$PRIMER_NUM_RETURN\n";

  if (@{$$primer3_base{$code}{'excluded'}})
  {
    print PR "EXCLUDED_REGION=";
    foreach my $e(@{$$primer3_base{$code}{'excluded'}})
    {
      print PR "$e ";
    }
    print PR "\n";
  }
  print PR "PRIMER_PRODUCT_SIZE_RANGE=$min-$max\n";
  print PR "=\n";

}
close PR;



}

#######################################################
sub run_primer3
{
my ($syst, $input, $output, $path) = @_;

my $primer3 = '';

if ($syst eq 'win')
{
	$primer3 = '"'.$path.'primer3_core.exe" <'.$input.' > '.$output;
}
elsif ($syst eq 'linux')
{
	$primer3 = $path.'primer3_core <'.$input.' > '.$output;
}
system $primer3;
}

#######################################################

sub read_results
{
 my ($primer3_out_file, $results, $condition, $target_ms_info) = @_;
 open_file_in($primer3_out_file, 'PRO');
 
my @seq_res = ();
while (my $l = <PRO>)
{
  push(@seq_res, $l);
 if ( $l eq "=\n")
 {
  if (scalar@seq_res >25) # at least one primer designed
  {

    my $code = shift@seq_res;
    chomp $code;
    $code =~ /(PRIMER_SEQUENCE_ID=)(.+?)(_[0-9]+,[0-9]+$)/;
    my $target_p = $3;
    $code = $2;
    $target_p =~ s/_//;
#    my @target_pos_l= split(',', $target_p);
#    print "$code $target_p\n";
    pop @seq_res;
 #   print "\n***********************\n@seq_res\n------------------------\n";

    my %pr_pair = initialise_pr_pair();
    foreach my $line (@seq_res)
    {
      chomp $line;
      my @line = split('=', $line);
      if ($line[0] =~ /PRIMER_PRODUCT_SIZE[_0-9]*$/) # end of the data on one primer pair
      {
        $line[0] =~ s/_[0-9]+$//;
        $pr_pair{$line[0]} = $line[1];
        $pr_pair{TARGET} = $target_p;
        add_info($results, \%pr_pair, $code, $condition, $target_ms_info);
        %pr_pair = initialise_pr_pair();
      }
      else
      {
        $line[0] =~ s/_[0-9]+$//;
        $line[0] =~ s/_[0-9]+_/_/;     
        if(exists $pr_pair{$line[0]})
        {
          $pr_pair{$line[0]} = $line[1];
        }
      }
    }# end foreach $line
  }# end if scalar @seq_res>
  @seq_res = ();
 }# end if $l eq "=\n"
}# end while
 
 
 close PRO;
 
}
##################################################
 
sub initialise_pr_pair
{
  my %pr_pair = ();
  $pr_pair{PRIMER_PAIR_PENALTY} = '';
  $pr_pair{PRIMER_LEFT_SEQUENCE} = '';
  $pr_pair{PRIMER_RIGHT_SEQUENCE} = '';
  $pr_pair{PRIMER_LEFT} = '';
  $pr_pair{PRIMER_RIGHT} = '';
  $pr_pair{PRIMER_LEFT_TM} = '';
  $pr_pair{PRIMER_RIGHT_TM} = '';
  $pr_pair{PRIMER_LEFT_END_STABILITY} = '';
  $pr_pair{PRIMER_RIGHT_END_STABILITY} = '';
  $pr_pair{PRIMER_PRODUCT_SIZE} = '';
  return %pr_pair;
}

##################################################

sub add_info
{
my ($results, $pr_pair, $code, $condition, $target_ms_info) = @_;
#  my %target_ms_info = (); # $target_ms_info{secode_first_pos} = [first_pos, length_bp, length_in_rn, p/c, mot_trans, ms_seq ];

my $code_pair = $$pr_pair{'PRIMER_LEFT'}.','.$$pr_pair{'PRIMER_RIGHT'};
delete $$pr_pair{'PRIMER_LEFT'};
delete $$pr_pair{'PRIMER_RIGHT'};

unless (exists $$results{$code}{$code_pair})# add info if primer pair is new to sequences
{
    $$pr_pair{'DESIGN'} = $condition;
    %{$$results{$code}{$code_pair}} = %$pr_pair;
#    print "$code $$results{$code}{$code_pair}{TARGET}\n";
}

}
##################################################################
sub write_primer_table
{
my ($results, $target_ms_info, $primer_file, $polym, $seq) = @_;
#  my %results = (); #$results{$code_seq}{left_pos,left_l,rigth_pos,rigth_l}{left_seq=>, ...}  
#  my %target_ms_info = (); # $target_ms{secode_first_pos} = [first_pos, length_bp, length_in_rn, p/c, mot_trans, ms_seq ];


open_file_out($primer_file, 'OUT');
print OUT "SEQUENCE_CODE;TARGET_REGION_FIRST_POS;TARGET_REGION_LENGTH_IN_BP;TARGET_MS_LENGTH_IN_REPEAT_NUMBER;PURE/COMPOUND;MOT_TRANS;TARGET_REGION_SEQ;POLYMORPH;";
print OUT "DESIGN;BEST_PR_FOR_SEQ;BEST_PR_FOR_TARGET_REGION;PENALTY;PCR_PRODUCT_SIZE;PRIMER_LEFT_SEQUENCE;PRIMER_RIGHT_SEQUENCE;PRIMER_LEFT_DIST_FROM_MS;PRIMER_RIGTH_DIST_FROM_MS;PRIMER_LEFT_FIRST_POS;PRIMER_LEFT_LENGTH;PRIMER_RIGHT_FIRST_POS;PRIMER_RIGHT_LENGTH;PRIMER_LEFT_TM;PRIMER_RIGHT_TM;PRIMER_LEFT_END_STABILITY;PRIMER_RIGHT_END_STABILITY;SEQUENCE\n";

foreach my $code (sort keys %$results)
{
  my $b_target = '';
  my $b_design = '';
  my $b_penalty = '';
  my %seq_pr_info = sort_primers($results, $code, $target_ms_info, \$b_target, \$b_design, \$b_penalty);
  my $best_seq = 1;
  
#  foreach my $target (sort {$a<=>$b} keys %seq_pr_info)
  foreach my $target (sort keys %seq_pr_info)
  {
    my $best_ms = 1;
    my $polymorph_data = 'NO';
    if(exists $$polym{$code})
    {
      $polymorph_data = check_overlap($$polym{$code}, $target);
    }
    foreach my $design (sort keys %{$seq_pr_info{$target}})
    {
      foreach my $penalty (sort {$a <=> $b} keys %{$seq_pr_info{$target}{$design}})
      {
        print OUT "$code;$$target_ms_info{$code.'_'.$target}[0];";
        print OUT "$$target_ms_info{$code.'_'.$target}[1];$$target_ms_info{$code.'_'.$target}[2];$$target_ms_info{$code.'_'.$target}[3];$$target_ms_info{$code.'_'.$target}[4];$$target_ms_info{$code.'_'.$target}[5];";
        print OUT "$polymorph_data;";
        print OUT "$design;";
        if (($best_seq == 1) and ($target eq $b_target) and ($design eq $b_design) and ($penalty == $b_penalty))
        {
          print OUT "1;";
          $best_seq = 0;
        }
        else
        {
          print OUT "0;";
        }
        if ($best_ms == 1)
        {
          print OUT "1;";
          $best_ms = 0;
        }
        else
        {
          print OUT "0;";
        }

        print OUT "$seq_pr_info{$target}{$design}{$penalty};";
        print OUT "$$seq{$code}\n";
      } # end foreach $penalty
    }  # end foreach $design
  } # end foreach $target
} # end foreach $code
close OUT;
}

##################################################################

sub sort_primers
{
my ($results, $code, $target_ms_info, $b_target, $b_design, $b_penalty) = @_;
my %seq_pr_info= ();

my @designs = (); # get list of design for seq
foreach my $prp (keys %{$$results{$code}})
{
    push (@designs, $$results{$code}{$prp}{DESIGN});# get list of design for seq
    my @pos = split(',', $prp);   
    my $rigth_dist = $pos[2] - $pos[3] - $$target_ms_info{$code.'_'.$$results{$code}{$prp}{TARGET}}[0] - $$target_ms_info{$code.'_'.$$results{$code}{$prp}{TARGET}}[1] +1 ;
    my $left_dist = $$target_ms_info{$code.'_'.$$results{$code}{$prp}{TARGET}}[0] - $pos[0] - $pos[1];

  $seq_pr_info{$$results{$code}{$prp}{TARGET}}{$$results{$code}{$prp}{DESIGN}}{$$results{$code}{$prp}{PRIMER_PAIR_PENALTY}} = "$$results{$code}{$prp}{PRIMER_PAIR_PENALTY}".';'."$$results{$code}{$prp}{PRIMER_PRODUCT_SIZE}".';'."$$results{$code}{$prp}{PRIMER_LEFT_SEQUENCE}".';'."$$results{$code}{$prp}{PRIMER_RIGHT_SEQUENCE}".';'."$left_dist".';'."$rigth_dist".';'."$pos[0]".';'."$pos[1]".';'."$pos[2]".';'."$pos[3]".';'."$$results{$code}{$prp}{PRIMER_LEFT_TM}".';'."$$results{$code}{$prp}{PRIMER_RIGHT_TM}".';'."$$results{$code}{$prp}{PRIMER_LEFT_END_STABILITY}".';'."$$results{$code}{$prp}{PRIMER_RIGHT_END_STABILITY}";
}

@designs = sort @designs;
$$b_design = $designs[0];

# detremine best target and score for seq
$$b_penalty = 10000000;
$$b_target = '';

my %best_design_seq = ();
foreach my $target (keys %seq_pr_info)
{
  foreach my $design (sort keys %{$seq_pr_info{$target}})
  {
    if ($design eq $$b_design)
    {
      my @pen = sort {$a<=>$b}(keys%{$seq_pr_info{$target}{$design}});
      if ($$b_penalty > $pen[0])
      {
        $$b_penalty = $pen[0];
        $$b_target = $target;
      }
    }
  }
}

return %seq_pr_info;
}
  

#####################################################################


sub make_polymorph_info
{
my ($file) = @_;
my %polym = ();
   # %polym is a hash of tables with cons seq code as keys and a table of motif_fpos_lapos as value

 open_file_in($file, 'IN');
 while (my $line = <IN>)
 {
    if ($line =~ />cons.[^\s]+/)
    {
      my $code = $&;
      chomp $line;
      $line =~ s/$code//;
      $line =~ s/\s//;
      $code =~ s/>//;
      unless ($line eq '')
      {
        my @line = split(';', $line);
        @{$polym{$code}} = @line;
      }
    }
 }
 close IN;
 return %polym;
}

##################################################

sub check_overlap
{
my ($polym_ref, $target) = @_;
my $poly_data = 'NO'; 
 #  $target_ms_ref if a ref to a table with [first_pos, length_bp, length_in_rn, p/c, mot_trans, ms_seq ];
  # polym_ref if a ref to a table with list of [motif_fpos_lpos]
  
  my @target_p_l = split(',', $target);
  foreach my $pol (@$polym_ref)
  {
    my @list = split('_', $pol);
    if ($target_p_l[0] >= $list[1] and $target_p_l[0] <= $list[2]) # first pos of target MS is between polymoprph pos
    {
      $poly_data = $pol;
      last;
    }
    if(($target_p_l[0] + $target_p_l[1]-1) >= $list[1] and ($target_p_l[0] + $target_p_l[1]-1)<= $list[2])# last pos of target MS is between polymoprph pos
    {
      $poly_data = $pol;
      last;    }
  }
  return $poly_data;
}

##################################################

sub make_fasta_file_with_designed_seq
{
 my ($results, $seq, $file) = @_;
 
 open_file_out($file, 'SEQ');
 my %seq_list = ();
 foreach my $code (sort keys %$results)
 {
      print SEQ ">$code\n$$seq{$code}\n";
 }
 
 close SEQ;
}

######################################################

sub define_param_number_pipe3
{
  my %param_list = (
      1=> ['syst', 'Operating system (win/linux):'], 
      2=> ['project_folder', 'Project folder:'], 
      3=> ['primer3_path', 'Pathway to Primer3 executables:'], 
      4=> ['primer3_version', 'Primer3 version (1/2):'],
      5=> ['del_files', 'Delete intermediate files (YES=1/NO=0):'],
      6=> ['pcr_min', 'Minimum length of PCR product for primer design:'],
      7=> ['pcr_max','Maximum length of PCR product for primer design:'],
      8=> ['pcr_step', 'PCR Product size interval:'],
      9=> ['PRIMER_MIN_SIZE', 'Minimum length of a primer (PRIMER_MIN_SIZE):'],
      10=> ['PRIMER_OPT_SIZE','Optimum length of a primer (PRIMER_OPT_SIZE):'],
      11=> ['PRIMER_MAX_SIZE','Maximum length of a primer (PRIMER_MAX_SIZE):'],
      12=> ['PRIMER_MIN_TM', 'Minimum melting temperature(Celsius) for a primer(PRIMER_MIN_TM):'],
      13=> ['PRIMER_OPT_TM','Optimum melting temperature(Celsius) for a primer(PRIMER_OPT_TM):'],
      14=> ['PRIMER_MAX_TM','Maximum melting temperature(Celsius) for a primer(PRIMER_MAX_TM):'],
      15=> ['PRIMER_MAX_DIFF_TM','Maximum acceptable difference between the melting temperatures of primers(PRIMER_MAX_DIFF_TM):'],
      16=> ['PRIMER_MIN_GC', 'Minimum percentage of Gs and Cs in any primer (PRIMER_MIN_GC):'],
      17=> ['PRIMER_OPT_GC_PERCENT','Optimum percentage of Gs and Cs in any primer (PRIMER_OPT_GC_PERCENT):'],
      18=> ['PRIMER_MAX_GC','Maximum percentage of Gs and Cs in any primer (PRIMER_MAX_GC):'],
      19=> ['PRIMER_GC_CLAMP','Number of consecutive Gs and Cs at the 3end of primer (PRIMER_GC_CLAMP):'],
      20=> ['PRIMER_SELF_ANY','The maximum allowable local alignment score for self- or pairwise-complementarity (PRIMER_SELF_ANY):'],
      21=> ['PRIMER_SELF_END','The maximum allowable 3end -anchored global alignment score for self- or pairwise-complementarity (PRIMER_SELF_END):'],
      22=> ['PRIMER_MAX_POLY_X', 'The maximum allowable length of a mononucleotide repeat in primer (PRIMER_MAX_POLY_X):'],
      23=> ['PRIMER_NUM_RETURN', 'The maximum number of primer pairs to return for each PCR product length interval (PRIMER_NUM_RETURN):']
      );
      
  return %param_list;
}


##################################################################

sub define_param_limits_pipe3
{
my ($param) = @_;
#hash of array [type(list, real, folder), error message, ['list of values/lower_upper limits)']
  my %param_limits = (
      'syst'=> ['list', 'Operating system must be win or linux', ['win', 'linux']], 
      'project_folder' => ['folder', "Cannot open the Project folder $$param{project_folder}"], 
      'primer3_path'=> ['folder', "Cannot open $$param{primer3_path} folder"], 
      'primer3_version'=> ['list', 'primer3_version must be 1 or 2', [1,2]],
      'del_files'=> ['list', 'del_files must be 0 or 1', [0,1]],
      'pcr_min' => ['real', 'Minimum PCR Product size must be between 40 and 10000', [40,10000]],
      'pcr_max' => ['real', 'Maximum PCR Product size must be between 40 and 10000', [40,10000]],
      'pcr_step' => ['real', 'PCR Product size interval must be between 20 and 10000', [20,10000]],
      'PRIMER_GC_CLAMP' => ['real', 'PRIMER_GC_CLAMP must be between 0 and 20', [0,20]],
      'PRIMER_OPT_SIZE' => ['real', 'PRIMER_OPT_SIZE must be between 1 and 50', [1,50]],
      'PRIMER_MIN_SIZE' => ['real', 'PRIMER_MIN_SIZE must be between 1 and 50', [1,50]],
      'PRIMER_MAX_SIZE' => ['real', 'PRIMER_MAX_SIZE must be between 1 and 50', [1,50]],
      'PRIMER_OPT_TM' => ['real', 'PRIMER_OPT_TM must be between 1 and 100', [1,100]],
      'PRIMER_MIN_TM' => ['real', 'PRIMER_MIN_TM must be between 1 and 100', [1,100]],
      'PRIMER_MAX_TM' => ['real', 'PRIMER_MAX_TM must be between 1 and 100', [1,100]],
      'PRIMER_MAX_DIFF_TM' => ['real', 'PRIMER_MAX_DIFF_TM must be between 0 and 100', [0,100]],
      'PRIMER_OPT_GC_PERCENT' => ['real', 'PRIMER_OPT_GC_PERCENT must be between 1 and 100', [1,100]],
      'PRIMER_MAX_GC' => ['real', 'PRIMER_MAX_GC must be between 1 and 100', [1,100]],
      'PRIMER_MIN_GC' => ['real', 'PRIMER_MIN_GC must be between 1 and 100', [1,100]],
      'PRIMER_SELF_ANY' => ['real', 'PRIMER_SELF_ANY must be between 1 and 100', [1,100]],
      'PRIMER_SELF_END' => ['real', 'PRIMER_SELF_END must be between 1 and 100', [1,100]],
      'PRIMER_MAX_POLY_X' => ['real', 'PRIMER_MAX_POLY_X must be between 1 and 10', [1,10]],
      'PRIMER_NUM_RETURN' => ['real', 'PRIMER_NUM_RETURN must be between 1 and 10', [1,10]]
      );

}
##################################################################
